@@ -6,8 +6,6 @@
 
 import numpy as np
 
-from joblib import Parallel, delayed
-from sklearn.utils import check_random_state
 from sklearn.linear_model import LinearRegression
 
 import functools
@@ -20,8 +18,8 @@ from . import _validation as val
 from .utils import ndiffs, is_constant, nsdiffs
 from ..utils import diff, is_iterable, check_endog
 from ..utils.metaestimators import if_has_delegate
-from .warnings import ModelFitWarning
 from ._context import AbstractContext, ContextType
+
 # Import as a namespace so we can mock
 from . import _auto_solvers as solvers
 from ..compat.numpy import DTYPE
@@ -48,52 +46,54 @@ class AutoARIMA(BaseARIMA):
     # Don't add the y, exog, etc. here since they are used in 'fit'
     __doc__ = _doc._AUTO_ARIMA_DOCSTR.format(
         y="",
-        exogenous="",
+        X="",
         fit_args="",
         return_valid_fits="",
         sarimax_kwargs=_doc._KWARGS_DOCSTR)
 
     # todo: someday store defaults somewhere else for single source of truth
-    def __init__(self,
-                 start_p=2,
-                 d=None,
-                 start_q=2,
-                 max_p=5,
-                 max_d=2,
-                 max_q=5,
-                 start_P=1,
-                 D=None,
-                 start_Q=1,
-                 max_P=2,
-                 max_D=1,
-                 max_Q=2,
-                 max_order=5,
-                 m=1,
-                 seasonal=True,
-                 stationary=False,
-                 information_criterion='aic',
-                 alpha=0.05,
-                 test='kpss',
-                 seasonal_test='ocsb',
-                 stepwise=True,
-                 n_jobs=1,
-                 start_params=None,
-                 trend=None,
-                 method='lbfgs',
-                 maxiter=50,
-                 offset_test_args=None,
-                 seasonal_test_args=None,
-                 suppress_warnings=False,
-                 error_action='trace',
-                 trace=False,
-                 random=False,
-                 random_state=None,
-                 n_fits=10,
-                 out_of_sample_size=0,
-                 scoring='mse',
-                 scoring_args=None,
-                 with_intercept="auto",
-                 **kwargs):
+    def __init__(
+        self,
+        start_p=2,
+        d=None,
+        start_q=2,
+        max_p=5,
+        max_d=2,
+        max_q=5,
+        start_P=1,
+        D=None,
+        start_Q=1,
+        max_P=2,
+        max_D=1,
+        max_Q=2,
+        max_order=5,
+        m=1,
+        seasonal=True,
+        stationary=False,
+        information_criterion='aic',
+        alpha=0.05,
+        test='kpss',
+        seasonal_test='ocsb',
+        stepwise=True,
+        n_jobs=1,
+        start_params=None,
+        trend=None,
+        method='lbfgs',
+        maxiter=50,
+        offset_test_args=None,
+        seasonal_test_args=None,
+        suppress_warnings=True,
+        error_action='trace',
+        trace=False,
+        random=False,
+        random_state=None,
+        n_fits=10,
+        out_of_sample_size=0,
+        scoring='mse',
+        scoring_args=None,
+        with_intercept="auto",
+        **kwargs,
+    ):
 
         self.start_p = start_p
         self.d = d
@@ -134,23 +134,14 @@ class AutoARIMA(BaseARIMA):
         self.scoring_args = scoring_args
         self.with_intercept = with_intercept
 
-        # TODO: pop out the deprecated vars for now, but remove in a later vsn
-        smx = kwargs.pop('sarimax_kwargs', None)
-        if smx:
-            kwargs = smx
-            warnings.warn("The 'sarimax_kwargs' keyword arg has been "
-                          "deprecated in favor of simply passing **kwargs. "
-                          "This will raise in future versions",
-                          DeprecationWarning)
-
         kwargs = _warn_for_deprecations(**kwargs)
         self.kwargs = kwargs
 
-    def fit(self, y, exogenous=None, **fit_args):
+    def fit(self, y, X=None, **fit_args):
         """Fit the auto-arima estimator
 
         Fit an AutoARIMA to a vector, ``y``, of observations with an
-        optional matrix of ``exogenous`` variables.
+        optional matrix of ``X`` variables.
 
         Parameters
         ----------
@@ -161,7 +152,7 @@ class AutoARIMA(BaseARIMA):
             one-dimensional array of floats, and should not contain any
             ``np.nan`` or ``np.inf`` values.
 
-        exogenous : array-like, shape=[n_obs, n_vars], optional (default=None)
+        X : array-like, shape=[n_obs, n_vars], optional (default=None)
             An optional 2-d array of exogenous variables. If provided, these
             variables are used as additional features in the regression
             operation. This should not include a constant or trend. Note that
@@ -172,50 +163,104 @@ class AutoARIMA(BaseARIMA):
             Any keyword arguments to pass to the auto-arima function.
         """
         sarimax_kwargs = {} if not self.kwargs else self.kwargs
+
         self.model_ = auto_arima(
-            y, exogenous=exogenous, start_p=self.start_p, d=self.d,
-            start_q=self.start_q, max_p=self.max_p, max_d=self.max_d,
-            max_q=self.max_q, start_P=self.start_P, D=self.D,
-            start_Q=self.start_Q, max_P=self.max_P, max_D=self.max_D,
-            max_Q=self.max_Q, max_order=self.max_order, m=self.m,
-            seasonal=self.seasonal, stationary=self.stationary,
+            y,
+            X=X,
+            start_p=self.start_p,
+            d=self.d,
+            start_q=self.start_q,
+            max_p=self.max_p,
+            max_d=self.max_d,
+            max_q=self.max_q,
+            start_P=self.start_P,
+            D=self.D,
+            start_Q=self.start_Q,
+            max_P=self.max_P,
+            max_D=self.max_D,
+            max_Q=self.max_Q,
+            max_order=self.max_order,
+            m=self.m,
+            seasonal=self.seasonal,
+            stationary=self.stationary,
             information_criterion=self.information_criterion,
-            alpha=self.alpha, test=self.test, seasonal_test=self.seasonal_test,
-            stepwise=self.stepwise, n_jobs=self.n_jobs,
-            start_params=self.start_params, trend=self.trend,
-            method=self.method, maxiter=self.maxiter,
+            alpha=self.alpha,
+            test=self.test,
+            seasonal_test=self.seasonal_test,
+            stepwise=self.stepwise,
+            n_jobs=self.n_jobs,
+            start_params=self.start_params,
+            trend=self.trend,
+            method=self.method,
+            maxiter=self.maxiter,
             offset_test_args=self.offset_test_args,
             seasonal_test_args=self.seasonal_test_args,
             suppress_warnings=self.suppress_warnings,
-            error_action=self.error_action, trace=self.trace,
-            random=self.random, random_state=self.random_state,
+            error_action=self.error_action,
+            trace=self.trace,
+            random=self.random,
+            random_state=self.random_state,
             n_fits=self.n_fits,
             return_valid_fits=False,  # only return ONE
-            out_of_sample_size=self.out_of_sample_size, scoring=self.scoring,
-            scoring_args=self.scoring_args, with_intercept=self.with_intercept,
-            sarimax_kwargs=sarimax_kwargs, **fit_args)
+            out_of_sample_size=self.out_of_sample_size,
+            scoring=self.scoring,
+            scoring_args=self.scoring_args,
+            with_intercept=self.with_intercept,
+            sarimax_kwargs=sarimax_kwargs,
+            **fit_args)
 
         return self
 
     @if_has_delegate("model_")
-    def predict_in_sample(self, exogenous=None, start=None,
-                          end=None, dynamic=False, return_conf_int=False,
-                          alpha=0.05, typ='levels'):
+    def predict_in_sample(
+        self,
+        X=None,
+        start=None,
+        end=None,
+        dynamic=False,
+        return_conf_int=False,
+        alpha=0.05,
+        typ='levels',
+    ):
         return self.model_.predict_in_sample(
-            exogenous=exogenous, start=start, end=end, dynamic=dynamic,
-            return_conf_int=return_conf_int, alpha=alpha, typ=typ)
+            X=X,
+            start=start,
+            end=end,
+            dynamic=dynamic,
+            return_conf_int=return_conf_int,
+            alpha=alpha,
+            typ=typ,
+        )
 
     @if_has_delegate("model_")
-    def predict(self, n_periods=10, exogenous=None,
-                return_conf_int=False, alpha=0.05):
+    def predict(
+        self,
+        n_periods=10,
+        X=None,
+        return_conf_int=False,
+        alpha=0.05,
+    ):
         return self.model_.predict(
-            n_periods=n_periods, exogenous=exogenous,
-            return_conf_int=return_conf_int, alpha=alpha)
+            n_periods=n_periods,
+            X=X,
+            return_conf_int=return_conf_int,
+            alpha=alpha,
+        )
 
     @if_has_delegate("model_")
-    def update(self, y, exogenous=None, maxiter=None, **kwargs):
+    def update(
+        self,
+        y,
+        X=None,
+        maxiter=None,
+        **kwargs,
+    ):
         return self.model_.update(
-            y, exogenous=exogenous, maxiter=maxiter, **kwargs)
+            y,
+            X=X,
+            maxiter=maxiter,
+            **kwargs
+        )
 
     @if_has_delegate('model_')
     def summary(self):
@@ -244,7 +289,7 @@ class StepwiseContext(AbstractContext):
     max_dur : int, optional (default=None)
         The maximum duration in seconds to try to find a best fit.
         When the cumulative fit duration exceeds this number, the
-        stepwiese process will stop and the best fit model at that
+        stepwise process will stop and the best fit model at that
         time will be returned. Please note that this is a soft limit.
 
     Notes
@@ -282,49 +327,51 @@ class StepwiseContext(AbstractContext):
         return ContextType.STEPWISE
 
 
-def auto_arima(y,
-               exogenous=None,
-               start_p=2,
-               d=None,
-               start_q=2,
-               max_p=5,
-               max_d=2,
-               max_q=5,
-               start_P=1,
-               D=None,
-               start_Q=1,
-               max_P=2,
-               max_D=1,
-               max_Q=2,
-               max_order=5,
-               m=1,
-               seasonal=True,
-               stationary=False,
-               information_criterion='aic',
-               alpha=0.05,
-               test='kpss',
-               seasonal_test='ocsb',
-               stepwise=True,
-               n_jobs=1,
-               start_params=None,
-               trend=None,
-               method='lbfgs',
-               maxiter=50,
-               offset_test_args=None,
-               seasonal_test_args=None,
-               suppress_warnings=False,
-               error_action='trace',
-               trace=False,
-               random=False,
-               random_state=None,
-               n_fits=10,
-               return_valid_fits=False,
-               out_of_sample_size=0,
-               scoring='mse',
-               scoring_args=None,
-               with_intercept="auto",
-               sarimax_kwargs=None,
-               **fit_args):
+def auto_arima(
+    y,
+    X=None,
+    start_p=2,
+    d=None,
+    start_q=2,
+    max_p=5,
+    max_d=2,
+    max_q=5,
+    start_P=1,
+    D=None,
+    start_Q=1,
+    max_P=2,
+    max_D=1,
+    max_Q=2,
+    max_order=5,
+    m=1,
+    seasonal=True,
+    stationary=False,
+    information_criterion='aic',
+    alpha=0.05,
+    test='kpss',
+    seasonal_test='ocsb',
+    stepwise=True,
+    n_jobs=1,
+    start_params=None,
+    trend=None,
+    method='lbfgs',
+    maxiter=50,
+    offset_test_args=None,
+    seasonal_test_args=None,
+    suppress_warnings=True,
+    error_action='trace',
+    trace=False,
+    random=False,
+    random_state=None,
+    n_fits=10,
+    return_valid_fits=False,
+    out_of_sample_size=0,
+    scoring='mse',
+    scoring_args=None,
+    with_intercept="auto",
+    sarimax_kwargs=None,
+    **fit_args,
+):
 
     # NOTE: Doc is assigned BELOW this function
 
@@ -357,9 +404,9 @@ def auto_arima(y,
                 raise ValueError('d & D must be None or a positive '
                                  'integer (>= 0)')
 
-    # check on n_iter
+    # check on n_fits
     if random and n_fits < 0:
-        raise ValueError('n_iter must be a positive integer '
+        raise ValueError('n_fits must be a positive integer '
                          'for a random search')
 
     # validate error action
@@ -372,7 +419,7 @@ def auto_arima(y,
     start = time.time()
 
     # copy array
-    y = check_endog(y, dtype=DTYPE)
+    y = check_endog(y, dtype=DTYPE, preserve_series=True)
     n_samples = y.shape[0]
 
     # the workhorse of the model fits
@@ -401,7 +448,7 @@ def auto_arima(y,
             solvers._sort_and_filter_fits(
                 fit_partial(
                     y,
-                    xreg=exogenous,
+                    X=X,
                     order=(0, 0, 0),
                     seasonal_order=(0, 0, 0, 0),
                     with_intercept=val.auto_intercept(
@@ -442,9 +489,9 @@ def auto_arima(y,
 
     # TODO: check rank deficiency, check for constant Xs, regress if necessary
     xx = y.copy()
-    if exogenous is not None:
-        lm = LinearRegression().fit(exogenous, y)
-        xx = y - lm.predict(exogenous)
+    if X is not None:
+        lm = LinearRegression().fit(X, y)
+        xx = y - lm.predict(X)
 
     # choose the order of differencing
     # is the TS stationary?
@@ -459,8 +506,8 @@ def auto_arima(y,
         D = nsdiffs(xx, m=m, test=seasonal_test, max_D=max_D,
                     **seasonal_test_args)
 
-        if D > 0 and exogenous is not None:
-            diffxreg = diff(exogenous, differences=D, lag=m)
+        if D > 0 and X is not None:
+            diffxreg = diff(X, differences=D, lag=m)
             # check for constance on any column
             if np.apply_along_axis(is_constant, arr=diffxreg, axis=0).any():
                 D -= 1
@@ -482,11 +529,11 @@ def auto_arima(y,
                          % D)
 
     # difference the exogenous matrix
-    if exogenous is not None:
+    if X is not None:
         if D > 0:
-            diffxreg = diff(exogenous, differences=D, lag=m)
+            diffxreg = diff(X, differences=D, lag=m)
         else:
-            diffxreg = exogenous
+            diffxreg = X
     else:
         # here's the thing... we're only going to use diffxreg if exogenous
         # was not None in the first place. However, PyCharm doesn't know that
@@ -497,13 +544,15 @@ def auto_arima(y,
     # determine/set the order of differencing by estimating the number of
     # orders it would take in order to make the TS stationary.
     if d is None:
-        d = ndiffs(dx,
-                   test=test,
-                   alpha=alpha,
-                   max_d=max_d,
-                   **offset_test_args)
+        d = ndiffs(
+            dx,
+            test=test,
+            alpha=alpha,
+            max_d=max_d,
+            **offset_test_args,
+        )
 
-        if d > 0 and exogenous is not None:
+        if d > 0 and X is not None:
             diffxreg = diff(diffxreg, differences=d, lag=1)
 
             # if any columns are constant, subtract one order of differencing
@@ -512,15 +561,7 @@ def auto_arima(y,
 
     # check differences (do we want to warn?...)
     if not suppress_warnings:  # TODO: context manager for entire block  # noqa: E501
-        if D >= 2:
-            warnings.warn("Having more than one seasonal differences is "
-                          "not recommended. Please consider using only one "
-                          "seasonal difference.", ModelFitWarning)
-        # if D is -1, this will be off, so we include the OR
-        elif D + d > 2 or d > 2:
-            warnings.warn("Having 3 or more differencing operations is not "
-                          "recommended. Please consider reducing the total "
-                          "number of differences.", ModelFitWarning)
+        val.warn_for_D(d=d, D=D)
 
     if d > 0:
         dx = diff(dx, differences=d, lag=1)
@@ -554,7 +595,7 @@ def auto_arima(y,
             solvers._sort_and_filter_fits(
                 fit_partial(
                     y,
-                    xreg=exogenous,
+                    X=X,
                     order=(0, d, 0),
                     seasonal_order=ssn,
                     with_intercept=with_intercept,
@@ -591,91 +632,80 @@ def auto_arima(y,
             raise ValueError('max_order must be None or a positive '
                              'integer (>= 0)')
 
-        # NOTE: pre-1.5.2, we started at start_p, start_q, etc. However, when
-        # using stepwise=FALSE in R, hyndman starts at 0. He only uses start_*
-        # when stepwise=TRUE.
-
-        # generate the set of (p, q, P, Q) FIRST, since it is contingent
-        # on whether or not the user is interested in a seasonal ARIMA result.
-        # This will reduce the search space for non-seasonal ARIMA models.
-        # loop p, q. Make sure to loop at +1 interval,
-        # since max_{p|q} is inclusive.
-        if seasonal:
-            gen = (
-                ((p, d, q), (P, D, Q, m))
-                for p in range(0, max_p + 1)
-                for q in range(0, max_q + 1)
-                for P in range(0, max_P + 1)
-                for Q in range(0, max_Q + 1)
-                if p + q + P + Q <= max_order
-            )
-        else:
-            # otherwise it's not seasonal and we don't need the seasonal pieces
-            gen = (
-                ((p, d, q), (0, 0, 0, 0))
-                for p in range(0, max_p + 1)
-                for q in range(0, max_q + 1)
-                if p + q <= max_order
-            )
-
-        # if we are fitting a random search rather than an exhaustive one, we
-        # will scramble up the generator (as a list) and only fit n_iter ARIMAs
-        if random:
-            random_state = check_random_state(random_state)
-
-            # make a list to scramble...
-            gen = random_state.permutation(list(gen))[:n_fits]
-
-        # get results in parallel
-        all_res = Parallel(n_jobs=n_jobs)(
-            delayed(fit_partial)(
-                y,
-                xreg=exogenous,
-                order=order,
-                seasonal_order=seasonal_order,
-                with_intercept=with_intercept,
-                **sarimax_kwargs)
-            for order, seasonal_order in gen)
-
-        # filter the non-successful ones and sort
-        sorted_res = solvers._sort_and_filter_fits(all_res)
+        search = solvers._RandomFitWrapper(
+            y=y,
+            X=X,
+            fit_partial=fit_partial,
+            d=d,
+            D=D,
+            m=m,
+            max_order=max_order,
+            max_p=max_p,
+            max_q=max_q,
+            max_P=max_P,
+            max_Q=max_Q,
+            random=random,
+            random_state=random_state,
+            n_fits=n_fits,
+            n_jobs=n_jobs,
+            seasonal=seasonal,
+            trace=trace,
+            with_intercept=with_intercept,
+            sarimax_kwargs=sarimax_kwargs,
+        )
 
     else:
-        # otherwise, we're fitting the stepwise algorithm...
         if n_samples < 10:
             start_p = min(start_p, 1)
             start_q = min(start_q, 1)
             start_P = start_Q = 0
 
-        # adjust to p, q, P, Q vals
-        p = start_p = min(start_p, max_p)
-        q = start_q = min(start_q, max_q)
-        P = start_P = min(start_P, max_P)
-        Q = start_Q = min(start_Q, max_Q)
+        # seed p, q, P, Q vals
+        p = min(start_p, max_p)
+        q = min(start_q, max_q)
+        P = min(start_P, max_P)
+        Q = min(start_Q, max_Q)
 
         # init the stepwise model wrapper
-        stepwise_wrapper = solvers._StepwiseFitWrapper(
-            y, xreg=exogenous, start_params=start_params, trend=trend,
-            method=method, maxiter=maxiter, fit_params=fit_args,
-            suppress_warnings=suppress_warnings, trace=trace,
-            error_action=error_action, out_of_sample_size=out_of_sample_size,
-            scoring=scoring, scoring_args=scoring_args, p=p, d=d, q=q,
-            P=P, D=D, Q=Q, m=m, start_p=start_p, start_q=start_q,
-            start_P=start_P, start_Q=start_Q, max_p=max_p, max_q=max_q,
-            max_P=max_P, max_Q=max_Q, seasonal=seasonal,
+        search = solvers._StepwiseFitWrapper(
+            y,
+            X=X,
+            start_params=start_params,
+            trend=trend,
+            method=method,
+            maxiter=maxiter,
+            fit_params=fit_args,
+            suppress_warnings=suppress_warnings,
+            trace=trace,
+            error_action=error_action,
+            out_of_sample_size=out_of_sample_size,
+            scoring=scoring,
+            scoring_args=scoring_args,
+            p=p,
+            d=d,
+            q=q,
+            P=P,
+            D=D,
+            Q=Q,
+            m=m,
+            max_p=max_p,
+            max_q=max_q,
+            max_P=max_P,
+            max_Q=max_Q,
+            seasonal=seasonal,
             information_criterion=information_criterion,
-            with_intercept=with_intercept, **sarimax_kwargs)
-
-        # do the step-through...
-        sorted_res = stepwise_wrapper.solve_stepwise()
+            with_intercept=with_intercept,
+            **sarimax_kwargs,
+        )
 
+    sorted_res = search.solve()
     return _return_wrapper(sorted_res, return_valid_fits, start, trace)
 
 
 # Assign the doc to the auto_arima func
 auto_arima.__doc__ = _doc._AUTO_ARIMA_DOCSTR.format(
     y=_doc._Y_DOCSTR,
-    exogenous=_doc._EXOG_DOCSTR,
+    X=_doc._EXOG_DOCSTR,
     fit_args=_doc._FIT_ARGS_DOCSTR,
     sarimax_kwargs=_doc._SARIMAX_ARGS_DOCSTR,
     return_valid_fits=_doc._VALID_FITS_DOCSTR
